//
//  iShuffleMain.m
//  iShuffle
//
//  Created by Matthias Lange on Thu Nov 13 2003.
//  Copyright (c) 2003 Maccent Software. All rights reserved.
//

#import "iShuffleMain.h"
#import "stdlib.h"
#import "myToolbarController.h"


@implementation iShuffleMain


NSArray *words;
NSString *word;

NSOpenPanel *openPanel;
NSSavePanel *savePanel;
NSMutableArray *fileTypes;

NSBundle *thisBundle;
NSString *imagePath, *text;
int i;
double elementCount = 0;
NSTextStorage *myTextStorage;

-(void)shuffle:(id)sender
{
    myTextStorage = [myTextView textStorage];
	text = [[NSString alloc] init];
	text = [myTextStorage string];
    words = [myTextStorage words];
    elementCount = [words count];
	[myProgressIndicator setMaxValue: elementCount];
	[self showProgressSheet];
    [myProgressIndicator startAnimation: window];
	[myProgressIndicator setIndeterminate: false];
    for(i=0; i < elementCount; i++)
    {
        word = (NSString *)[[words objectAtIndex: i] string];
        if([word length] > 3)
            [myTextStorage replaceCharactersInRange: [text rangeOfString: word] withString: [self mix: word]];
		[myProgressIndicator incrementBy: 1.0];
    }
    [self hideProgressSheet];
	elementCount = 0;
}

-(NSString *)mix:(NSString *)word
{
    int j, wordLength = [word length], mod, randNumber;
    unichar buffer[wordLength], c;
    NSRange myRange;
    NSString *mixed;
    
    myRange.length = wordLength;
    myRange.location = 0;
    [word getCharacters: buffer range: myRange];
    for(j=1; j < wordLength-1; j++)
    {
        mod = wordLength-j-1;
        randNumber = rand() % mod;
        c = buffer[j+randNumber];
        buffer[j+randNumber] = buffer[j];
        buffer[j] = c;
    }
    
    mixed = [[NSString alloc] initWithCharacters: buffer length: [word length]];
    return [mixed autorelease];
}

-(IBAction)openDocument:(id)sender
{
    openPanel = [NSOpenPanel openPanel];
    
    [openPanel beginSheetForDirectory: nil file: nil types: (NSArray *)fileTypes modalForWindow: [myTextView window] modalDelegate: self 
        didEndSelector: @selector(openPanelDidEnd:returnCode:contextInfo:) contextInfo: nil];
}

-(IBAction)saveDocument:(id)sender
{
    savePanel = [NSSavePanel savePanel];
    
    [savePanel setAllowedFileTypes: (NSArray *)fileTypes];
    
    [savePanel beginSheetForDirectory: nil file: nil modalForWindow: [myTextView window] modalDelegate: self 
        didEndSelector: @selector(savePanelDidEnd:returnCode:contextInfo:) contextInfo: nil];
    
}

- (void)openPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    NSData *myData;
    if(returnCode == NSOKButton)
    {
        myData = [NSData dataWithContentsOfFile: [openPanel filename]];
		if([[[openPanel filename] pathExtension] isEqualToString: @"txt"])
			[myTextView setString: [NSString stringWithContentsOfFile: [openPanel filename]]];
		else
			[myTextView replaceCharactersInRange: NSMakeRange(0, [[myTextView string] length]) withRTF: myData];
    }
}

-(void)savePanelDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    NSData *myData;
    
    myData = [myTextView RTFFromRange: NSMakeRange(0, [[myTextView string] length])];
    if(returnCode = NSOKButton)
    {
        [myData writeToFile:[savePanel filename] atomically: YES];
    }
}

- (void)awakeFromNib
{
    thisBundle = [NSBundle bundleForClass:[self class]];
    imagePath = [[NSString alloc] init];
    imagePath = [thisBundle pathForResource:@"Shuffle" ofType:@"icns"];
    NSImage *myImage = [[NSImage alloc] initWithContentsOfFile: imagePath];
    
    fileTypes = [[NSMutableArray alloc] init];
    items=[[NSMutableDictionary alloc] init];
    
    [fileTypes addObject: (id)@"rtf"];
	[fileTypes addObject: (id)@"txt"];
	//[fileTypes addObject: NSFileTypeForHFSTypeCode( 'TEXT' )];
    NSString *name=[[NSString alloc] initWithFormat:@"Shuffle"];
    NSToolbarItem *item=[[NSToolbarItem alloc] initWithItemIdentifier:name];

    [item setPaletteLabel:name]; // name for the "Customize Toolbar" sheet
    [item setLabel:name]; // name for the item in the toolbar
    [item setToolTip:[NSString stringWithFormat:@"Shuffle text"]]; // tooltip
    [item setTarget:self]; // what should happen when it's clicked
    [item setAction:@selector(shuffle:)];
    [item setImage: myImage];
    
    [items setObject:item forKey:name]; // add to toolbar list
    
           
    [item release];
    [name release];
//    [imagePath release];
    
    toolbar=[[NSToolbar alloc] initWithIdentifier:@"tooltest"]; // identifier has to be unique per window type
    [toolbar setDelegate:self];
    [toolbar setAllowsUserCustomization:YES];
    [toolbar setAutosavesConfiguration:YES];
    
    [window setToolbar:toolbar];
    
    [window makeKeyAndOrderFront:nil];
}

- (void)dealloc
{
    [toolbar release];
    [items release];
    [imagePath release];
    [thisBundle release];
    [words release];
    [super dealloc];
}

- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar itemForItemIdentifier:(NSString *)itemIdentifier willBeInsertedIntoToolbar:(BOOL)flag
{
    return [items objectForKey:itemIdentifier];
}

- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar*)toolbar
{
    return [[items allKeys] subarrayWithRange:NSMakeRange(0,1)];
}

- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar*)toolbar
{
    return [items allKeys];
}

- (int)count
{
    return [items count];
}

- (IBAction)customize:(id)sender
{
    [toolbar runCustomizationPalette:sender];
}

- (IBAction)showhide:(id)sender
{
    [toolbar setVisible:![toolbar isVisible]];
}

- (void)showProgressSheet
{
    [NSApp beginSheet: progressSheet modalForWindow: window modalDelegate:self didEndSelector:NULL contextInfo:nil];
}

- (void)hideProgressSheet
{
    [progressSheet orderOut:nil];
    [NSApp endSheet: progressSheet];
}

@end
